package dk.nsp.idsas.security;

import dk.nsp.idsas.security.impl.IdsasActorEnricherImpl;
import dk.nsp.idsas.security.impl.IdsasActorProviderImpl;
import dk.nsp.idsas.security.impl.IdsasActorTransformerImpl;
import dk.nsp.idsas.security.impl.IdsasActorValidatorImpl;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.FaultCode;
import dk.nsp.idsas.util.IdsasException;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.sds.nsp.security.SecurityContext;

import java.util.function.Supplier;
import org.apache.log4j.Logger;

public class IdsasActorBuilder {
    private static final Logger logger = Logger.getLogger(IdsasActorBuilder.class);

    private final Supplier<SecurityContext> securitySupplier;

    public IdsasActorBuilder(Supplier<SecurityContext> securitySupplier) {
        this.securitySupplier = securitySupplier;
    }

    public IdsasActor buildActor() throws IdsasSecurityException {

        IdsasActorProvider provider = new IdsasActorProviderImpl(securitySupplier);
        IdsasActorTransformer transformer = new IdsasActorTransformerImpl();
        IdsasActorEnricher enricher = new IdsasActorEnricherImpl();
        IdsasActorValidator validator = new IdsasActorValidatorImpl(securitySupplier);

        IdsasActor actor = provider.getActor();
        actor = transformer.getActor(actor);
        actor = enricher.getActor(actor);
        actor = validator.getActor(actor);

        return actor;
    }

    public IdsasActor buildOrgBlurringActor() {
        IdsasActor actor;
        try {
            actor = buildActor();
            if (!actor.getActorType().equals(IdsasActor.IdsasActorType.SYSTEM)) {
                throw new IdsasSecurityException(ErrorMessages.userTypeNotSupported(
                        actor.getActorType().getDisplayableText()), FaultCode.CREATE_ORG_BLURRING_ERROR, true);
            }
            if (!actor.getOrganizationIdType().equalsIgnoreCase("CVR")) {
                throw new IdsasSecurityException(ErrorMessages.organizationIdTypeNotSupported(
                        actor.getOrganizationIdType()), FaultCode.CREATE_ORG_BLURRING_ERROR, true);
            }
        } catch (IdsasException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }
        return actor;
    }
}
