package dk.nsp.idsas.ws._20240124;

import com.sun.xml.ws.developer.SchemaValidation;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.service.IdsasServiceException;
import dk.nsp.idsas.service.OrgBlurringService;
import dk.nsp.idsas.util.FaultCode;
import dk.nsp.idsas.util.IdsasException;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2024._01._24.*;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;
import java.time.ZonedDateTime;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name="IdsasRegistration20240124", urlPatterns={"/20240124/registration"})
@WebService(serviceName = "IdsasRegistration20240124", wsdlLocation = "wsdl/20240124/idsas-registration.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2024/01/24/",
        portName = "IdsasRegistrationPort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation
public class IdsasRegistrationWS extends IdsasAbstractWS implements IdsasRegistrationPortType {
    private static final Logger logger = Logger.getLogger(IdsasRegistrationWS.class);

    @Autowired
    private IdsasActorBuilder idsasActorBuilder;

    @Autowired
    private BlurringService blurringService;

    @Autowired
    private OrgBlurringService orgBlurringService;

    @Override
    public String createBlurring(CreateBlurringRequest createBlurringRequest) {
        logger.trace("Enter create blurring");

        try {
            IdsasActor actor = idsasActorBuilder.buildActor();
            String organizationId = actor.getOrganizationId();
            String organizationIdType = actor.getOrganizationIdType();
            String patientIdType = createBlurringRequest.getPatientId().getClassification().value();
            String patientId = createBlurringRequest.getPatientId().getId();
            ZonedDateTime endDateTime = createBlurringRequest.getEndDateTime().toGregorianCalendar().toZonedDateTime();

            blurringService.createBlurring(organizationId, organizationIdType, patientId, patientIdType, endDateTime);
        } catch (IdsasException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        } catch (Exception e) {
            logger.error("Failed to create blurring", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error",
                    FaultCode.CREATE_BLURRING_ERROR.getValue());
        }

        return "OK";
    }

    @Override
    public String createOrgBlurring(CreateOrgBlurringRequest createOrgBlurringRequest) {
        IdsasActor actor = idsasActorBuilder.buildOrgBlurringActor();

        try {
            orgBlurringService.createOrgBlurring(actor.getOrganizationId(),
                    createOrgBlurringRequest.getOrg().getId(), createOrgBlurringRequest.getOrg().getClassification().value());
        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }

        return "OK";
    }

    @Override
    public String removeOrgBlurring(RemoveOrgBlurringRequest removeOrgBlurringRequest) {
        IdsasActor actor = idsasActorBuilder.buildOrgBlurringActor();

        try {
            orgBlurringService.removeOrgBlurring(actor.getOrganizationId(), removeOrgBlurringRequest.getId(), removeOrgBlurringRequest.getClassification().value());
        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }

        return "OK";
    }
}
